#!/usr/bin/perl

# exon_builder2 - This program scans a SAM file of short reads to identify possible exons via tiling the reads
# Written by Kenneth Watanabe, University of Nevada Las Vegas 
# copyright (c) 2014


use DBI;
use Tk;
use Tk::ROText;
use Tk::BrowseEntry;
use File::Basename;
use Statistics::Descriptive;

require "connectDB.pl";

our $uname;
our $passwd;

$mw = MainWindow->new(-title => "Generate Exons from Short Reads");
# In the geometry function, the 1000x400 is the size of the window 1000 pixels horizontal, 400 pixels vertical
# +400+200 is the position on the screen where the window is to be pasted, 300 pixels from the left of the screen, 200 pixels from the top 
$mw -> geometry('1000x400+300+160');

# Create frames in the main window 
$f1 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)  ->pack(-fill => 'x');
$f2 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)  ->pack(-fill => 'x');
$f3 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)  ->pack(-fill => 'x');
$f4 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)  ->pack(-fill => 'x');
$f5 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)  ->pack(-fill => 'x');
$f6 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)  ->pack(-fill => 'x');
$f7 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)  ->pack(-fill => 'x');
$f8 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)  ->pack(-fill => 'x');
$fbot = $mw->Frame() ->pack(-side=>'bottom',-fill => 'x');

# Create an go button
$go = $fbot->Button(-text => 'Go', -command => \&do_search)
    ->pack(-side => 'left');
# Create an exit button
$ex = $fbot->Button(-text => 'Exit', -command => \&do_exit)
    ->pack(-side => 'right');
# Create an edit button
$ed = $fbot->Button(-text => 'View Results', -command => \&do_edit)
    ->pack();


($name,$path,$suffix)=fileparse($0,"\.[^.]*");
$default_file = $ENV{'HOME'} . "/perl/".$name.".default";
open(DEFAULT_VALS,$default_file);

# Creates Browse Entry pulldown box in frame f1
$outfile = <DEFAULT_VALS>;
$outfile =~ s/\s+$//; #remove trailing spaces
if ($outfile ne "") { ($name,$path,$suffix) = fileparse($outfile,"\.[^.]*"); }
$f1 -> Label(-text=>"Output File: ") -> pack(-side => 'left');  # creates a label in frame $f
$be1 = $f1->BrowseEntry(-variable => \$outfile,-width=>50)->pack(-side => 'left');
$f1 -> Button(-text=>'Search',-command =>\&get_file1) -> pack(-side=>'left');

# Creates Browse Entry pulldown box in frame f1
$outfile2 = <DEFAULT_VALS>;
$outfile2 =~ s/\s+$//; #remove trailing spaces
$f2 -> Label(-text=>"File for reads with introns: ") -> pack(-side => 'left');  # creates a label in frame $f
$be2 = $f2->BrowseEntry(-variable => \$outfile2,-width=>50)->pack(-side => 'left');
$f2 -> Button(-text=>'Search',-command =>\&get_file2) -> pack(-side=>'left');

$samptype = <DEFAULT_VALS>;
$samptype =~ s/\s+$//; #remove trailing spaces
$f3 -> Label(-text=>"Sample Type: ") -> pack(-side => 'left');  # creates a label in frame $f
$be3 = $f3->BrowseEntry(-variable => \$samptype,-width=>50)->pack(-side => 'left');


$min_reads_per_kb = <DEFAULT_VALS>;
$min_reads_per_kb =~ s/\s+$//; #remove trailing spaces
$f4 -> Label(-text=>"Min reads/kb of exon: ") -> pack(-side => 'left');  # creates a label in frame $f
$be4 = $f4->BrowseEntry(-variable => \$min_reads_per_kb,-width=>50)->pack(-side => 'left');

$dbase_name = <DEFAULT_VALS>;
$dbase_name =~ s/\s+$//;
$f5 -> Label(-text=>"Database name: ") -> pack(-side => 'left'); 
$be5 = $f5->BrowseEntry(-variable => \$dbase_name,-width=>50)->pack(-side => 'left');


$short_read_table = <DEFAULT_VALS>;
$short_read_table =~ s/\s+$//; #remove trailing spaces
$f6 -> Label(-text=>"Short read table: ") -> pack(-side => 'left');  # creates a label in frame $f
$be6 = $f6->BrowseEntry(-variable => \$short_read_table,-width=>50)->pack(-side => 'left');

# Initialize browse entry with some default values
$host = "localhost";
$dbh = DBI -> connect("DBI:mysql:information_schema:$host",$uname,$passwd);
$query1 = "select table_name from tables where table_schema = \"$dbase_name\" order by table_name ASC";
$sqlquery1 = $dbh->prepare($query1) or die "Cannot prepare query \n";
$rv1 = $sqlquery1->execute or die "Cannot execute query \n";
@row = $sqlquery1->fetchrow_array();
while (defined $row[0])
{
	$be6->insert('end', $row[0]);
	@row = $sqlquery1->fetchrow_array();
}
$sqlquery1->finish;




$organism = <DEFAULT_VALS>;
$organism =~ s/\s+$//; #remove trailing spaces
$f7 -> Label(-text=>"Organism: ") -> pack(-side => 'left');  # creates a label in frame $f
$be7 = $f7->BrowseEntry(-variable => \$organism,-width=>50)->pack(-side => 'left');


$read_length = <DEFAULT_VALS>;
$read_length =~ s/\s+$//; #remove trailing spaces
$f8 -> Label(-text=>"Read length: ") -> pack(-side => 'left');  # creates a label in frame $f
$be8 = $f8->BrowseEntry(-variable => \$read_length,-width=>50)->pack(-side => 'left');


# Initialize browse entry with some default values
$be3->insert('end', "control");
$be3->insert('end', "ABA");
$be3->insert('end', "GA");
$be3->insert('end', "ABA+GA");
$be3->insert('end', "All");

# Initialize browse entry with some default values
$be7->insert('end', "Oryza sativa");
$be7->insert('end', "Arabidopsis thaliana");
$be7->insert('end', "Caenorhabditis elegans");
$be7->insert('end', "Drosophila melanogaster");
$be7->insert('end', "Yeast");
$be7->insert('end', "Test Genome");

close(DEFAULT_VALS);

MainLoop;

###############################################################################################
# This routine is called when the "Search" button is clicked

sub get_file1 {
$outfile = $mw -> getOpenFile(-initialdir => $path);
if ($outfile ne "") { ($name,$path,$suffix) = fileparse($outfile,"\.[^.]*"); }
}

sub get_file2 {
$outfile2 = $mw -> getOpenFile(-initialdir => $path);
}
###############################################################################################

sub do_exit {
# Put commands here to be executed on exit
  exit;
}


sub save_defaults {
open(DEFAULT_VALUES,">$default_file");
$outfile =~ s/\s+$//; #remove trailing spaces
$outfile2 =~ s/\s+$//; #remove trailing spaces
$samptype =~ s/\s+$//; #remove trailing spaces
$min_reads_per_kb =~ s/\s+$//; #remove trailing spaces
$dbase_name =~ s/\s+$//; #remove trailing spaces
$short_read_table =~ s/\s+$//; #remove trailing spaces
$organism =~ s/\s+$//; #remove trailing spaces
$read_length =~ s/\s+$//; #remove trailing spaces
print DEFAULT_VALUES "$outfile\n";
print DEFAULT_VALUES "$outfile2\n";
print DEFAULT_VALUES "$samptype\n";
print DEFAULT_VALUES "$min_reads_per_kb\n";
print DEFAULT_VALUES "$dbase_name\n";
print DEFAULT_VALUES "$short_read_table\n";
print DEFAULT_VALUES "$organism\n";
print DEFAULT_VALUES "$read_length\n";
close(DEFAULT_VALUES);
}

###############################################################################################

sub do_edit {
$edit_cmd = "gedit " . $outfile;
system $edit_cmd;
}
###############################################################################################

sub do_search
{

$database = "deep_seq";
$host = "localhost";
$dbh = DBI -> connect("DBI:mysql:$database:$host",$uname,$passwd);
&save_defaults;

$debug = 0;

open(OUT_FILE2,">$outfile2") || die "Cannot open output file $outfile2\n";
open(OUT_FILE,">$outfile") || die "Cannot open output file $outfile\n";
print OUT_FILE "Chromosome\tStart Position\tEnd Position\tExon Length\tRead Count\n";

if ($organism eq "Oryza sativa")
	{ $num_chroms = 12;
	  @chrom_names = ("chr01","chr02","chr03","chr04","chr05","chr06","chr07","chr08","chr09","chr10","chr11","chr12"); }
elsif ($organism eq "Arabidopsis thaliana")
	{ $num_chroms = 5; 
	  @chrom_names = ("chr1","chr2","chr3","chr4","chr5"); }
elsif ($organism eq "Caenorhabditis elegans")
	{ $num_chroms = 7; 
	  @chrom_names = ("chr01","chr02","chr03","chr04","chr05","chrX","chrM"); }
elsif ($organism eq "Drosophila melanogaster")
	{ $num_chroms = 15; 
	  @chrom_names = ("2L","2R","2LHet","2RHet","3L","3R","3LHet","3RHet","4","X","XHet","YHet","U","Uextra","mito"); }
elsif ($organism eq "Yeast")
	{ $num_chroms = 16;
	  @chrom_names = ("chr01","chr02","chr03","chr04","chr05","chr06","chr07","chr08","chr09","chr10","chr11","chr12","chr13","chr14","chr15","chr16"); }
elsif ($organism eq "Test Genome")
	{ $num_chroms = 1; 
	  @chrom_names = ("chr01"); }







else
	{ $answer = $mw -> messageBox(-message=>"Invalid organism",-type=>'Ok'); 
	  return; }

$chrom_count = 0;
while ($chrom_count < $num_chroms)
{
$chrom_name = $chrom_names[$chrom_count];

if ($samptype eq "All")
	{
	$query1 = "select chromosome,start_position,cigar,read_id from $short_read_table where (chromosome = \"$chrom_name\") order by start_position ASC";
	}
else
	{
	$query1 = "select chromosome,start_position,cigar,read_id from $short_read_table where (sample_type = \"$samptype\") and (chromosome = \"$chrom_name\") order by start_position ASC";
	}


$sqlquery1 = $dbh->prepare($query1) or die "Cannot prepare query \n";
$rv1 = $sqlquery1->execute or die "Cannot execute query \n";
@row = $sqlquery1->fetchrow_array();

$chrom = $row[0];
$gene_start = $row[1];
$cigar = $row[2];
$read_id = $row[3];
$gene_end = $row[1] + $read_length;
$read_count = 1;
$gene_count = 0;


while (defined $row[0])
{
	@row = $sqlquery1->fetchrow_array();
	$chrom = $row[0];
	$read_start = $row[1];
	$cigar = $row[2];
	$read_id = $row[3];
	while ((defined $row[0]) and ($read_start <= $gene_end))
	{	
		$read_count++;
		if (index($cigar,"N") < 0)		# ignore reads that span an intron
			{ $gene_end = $read_start + $read_length; }
		else
			{ print OUT_FILE2 "$read_id\t$chrom\t$read_start\t$cigar\n"; }
		@row = $sqlquery1->fetchrow_array();
		$chrom = $row[0];
		$read_start = $row[1];
		$cigar = $row[2];
		$read_id = $row[3];
	}


	$gene_length = $gene_end - $gene_start;
	$reads_per_kb = $read_count/($gene_length/1000);

	if ($reads_per_kb >= $min_reads_per_kb)
	{
		print OUT_FILE "$chrom_name\t$gene_start\t$gene_end\t$gene_length\t$read_count\n";

		if (($gene_count % 1000) == 0)
		{
			print "$gene_count\t$chrom_name\t$gene_start\t$gene_end\n";
		}
		$gene_count++;
	}

	$gene_start = $row[1];
	$gene_end = $row[1] + 50;
	$read_count = 1;
} #endwhile

$chrom_count++;
}

close(OUT_FILE);
close(OUT_FILE2);

print "Finished\n";
}

