#!/usr/bin/perl

# exons_from_junctions - This program relies on overlapping junctions to identify exons that are shorter than the length of a single read
# Written by Kenneth Watanabe, University of Nevada Las Vegas
# Copywrite (c) 2014

use DBI;
use Tk;
use Tk::ROText;
use Tk::BrowseEntry;
use File::Basename;
use Statistics::Descriptive;

require "connectDB.pl";

our $uname;
our $passwd;

$mw = MainWindow->new(-title => "Generate Exons from Short Reads");
# In the geometry function, the 600x300 is the size of the window 800 pixels horizontal, 300 pixels vertical
# +400+200 is the position on the screen where the window is to be pasted, 300 pixels from the left of the screen, 200 pixels from the top 
$mw -> geometry('800x300+300+160');

# Create frame f1 in the main window 
$f1 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f2 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f3 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f4 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f5 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f6 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$fbot = $mw->Frame() ->pack(-side=>'bottom',-fill => 'x');

# Create an go button
$go = $fbot->Button(-text => 'Go', -command => \&do_search)
    ->pack(-side => 'left');
# Create an exit button
$ex = $fbot->Button(-text => 'Exit', -command => \&do_exit)
    ->pack(-side => 'right');
# Create an edit button
$ed = $fbot->Button(-text => 'View Results', -command => \&do_edit)
    ->pack();

($name,$path,$suffix)=fileparse($0,"\.[^.]*");
$default_file = $ENV{'HOME'} . "/perl/".$name.".default";
open(DEFAULT_VALS,$default_file);

# Creates Browse Entry pulldown box in frame f1
$infile = <DEFAULT_VALS>;
$infile =~ s/\s+$//; #remove trailing spaces
if ($infile ne "") { ($name,$path1,$suffix) = fileparse($infile,"\.[^.]*"); }
$f1 -> Label(-text=>"Input File: ") -> pack(-side => 'left');  # creates a label in frame $f
$be1 = $f1->BrowseEntry(-variable => \$infile,-width=>50)->pack(-side => 'left');
$f1 -> Button(-text=>'Search',-command =>\&get_file1) -> pack(-side=>'left');


$outfile = <DEFAULT_VALS>;
$outfile =~ s/\s+$//; #remove trailing spaces
if ($outfile ne "") { ($name,$path2,$suffix) = fileparse($outfile,"\.[^.]*"); }
$f2 -> Label(-text=>"Output File: ") -> pack(-side => 'left');  # creates a label in frame $f
$be2 = $f2->BrowseEntry(-variable => \$outfile,-width=>50)->pack(-side => 'left');
$f2 -> Button(-text=>'Search',-command =>\&get_file2) -> pack(-side=>'left');

$samptype = <DEFAULT_VALS>;
$samptype =~ s/\s+$//; #remove trailing spaces
$f3 -> Label(-text=>"Sample Type: ") -> pack(-side => 'left');  # creates a label in frame $f
$be3 = $f3->BrowseEntry(-variable => \$samptype,-width=>50)->pack(-side => 'left');


$min_reads = <DEFAULT_VALS>;
$min_reads =~ s/\s+$//; #remove trailing spaces
$f4 -> Label(-text=>"Min junctions: ") -> pack(-side => 'left');  # creates a label in frame $f
$be4 = $f4->BrowseEntry(-variable => \$min_reads,-width=>50)->pack(-side => 'left');

$short_read_table = <DEFAULT_VALS>;
$short_read_table =~ s/\s+$//; #remove trailing spaces
$f5 -> Label(-text=>"Junction Ends table: ") -> pack(-side => 'left');  # creates a label in frame $f
$be5 = $f5->BrowseEntry(-variable => \$short_read_table,-width=>50)->pack(-side => 'left');


# Initialize browse entry with some default values
$host = "localhost";
$dbase_name = "deep_seq";
$dbh = DBI -> connect("DBI:mysql:information_schema:$host",$uname,$passwd);
$query1 = "select table_name from tables where table_schema = \"$dbase_name\" order by table_name ASC";
$sqlquery1 = $dbh->prepare($query1) or die "Cannot prepare query \n";
$rv1 = $sqlquery1->execute or die "Cannot execute query \n";
@row = $sqlquery1->fetchrow_array();
while (defined $row[0])
{
	$be5->insert('end', $row[0]);
	@row = $sqlquery1->fetchrow_array();
}
$sqlquery1->finish;


$organism = <DEFAULT_VALS>;
$organism =~ s/\s+$//; #remove trailing spaces
$f6 -> Label(-text=>"Organism: ") -> pack(-side => 'left');  # creates a label in frame $f
$be6 = $f6->BrowseEntry(-variable => \$organism,-width=>50)->pack(-side => 'left');



# Initialize browse entry with some default values
$be3->insert('end', "control");
$be3->insert('end', "ABA");
$be3->insert('end', "GA");
$be3->insert('end', "ABA+GA");
$be3->insert('end', "All");

# Initialize browse entry with some default values
$be6->insert('end', "Oryza sativa");
$be6->insert('end', "Arabidopsis thaliana");
$be6->insert('end', "Caenorhabditis elegans");
$be6->insert('end', "Drosophila melanogaster");
$be6->insert('end', "Yeast");
$be6->insert('end', "Test Genome");

close(DEFAULT_VALS);

MainLoop;

###############################################################################################
# This routine is called when the "Search" button is clicked

sub get_file1 {
$infile = $mw -> getOpenFile(-initialdir => $path1);
}

sub get_file2 {
$outfile = $mw -> getOpenFile(-initialdir => $path2);
}

###############################################################################################

sub do_exit {
# Put commands here to be executed on exit
  exit;
}


sub save_defaults {
open(DEFAULT_VALUES,">$default_file");
$infile =~ s/\s+$//; #remove trailing spaces
$outfile =~ s/\s+$//; #remove trailing spaces
$samptype =~ s/\s+$//; #remove trailing spaces
$min_reads =~ s/\s+$//; #remove trailing spaces
$short_read_table =~ s/\s+$//; #remove trailing spaces
$organism =~ s/\s+$//; #remove trailing spaces
print DEFAULT_VALUES "$infile\n";
print DEFAULT_VALUES "$outfile\n";
print DEFAULT_VALUES "$samptype\n";
print DEFAULT_VALUES "$min_reads\n";
print DEFAULT_VALUES "$short_read_table\n";
print DEFAULT_VALUES "$organism\n";
close(DEFAULT_VALUES);
}

###############################################################################################

sub do_edit {
$edit_cmd = "gedit " . $outfile;
system $edit_cmd;
}
###############################################################################################

sub do_search
{

$database = "deep_seq";
$host = "localhost";
$dbh = DBI -> connect("DBI:mysql:$database:$host",$uname,$passwd);
&save_defaults;

$debug = 0;

open(IN_FILE,$infile);
$header_line = <IN_FILE>;
$line = <IN_FILE>;
@row = split("\t",$line);
$chrom1 = $row[0];
$start_pos1 = $row[1];
$end_pos1 = $row[2];
$exon_len1 = $row[3];
$read_count1 = $row[4];
$read_count1 =~ s/\s+$//; #remove trailing spaces

open(OUT_FILE,">$outfile");
print OUT_FILE "Chromosome\tStart Position\tEnd Position\tExon Length\tRead Count\n";

if ($organism eq "Oryza sativa")
	{ $num_chroms = 12;
	  @chrom_names = ("chr01","chr02","chr03","chr04","chr05","chr06","chr07","chr08","chr09","chr10","chr11","chr12"); }
elsif ($organism eq "Arabidopsis thaliana")
	{ $num_chroms = 5; 
	  @chrom_names = ("chr1","chr2","chr3","chr4","chr5"); }
elsif ($organism eq "Caenorhabditis elegans")
	{ $num_chroms = 7; 
	  @chrom_names = ("chr01","chr02","chr03","chr04","chr05","chrX","chrM"); }
elsif ($organism eq "Drosophila melanogaster")
	{ $num_chroms = 15; 
	  @chrom_names = ("2L","2R","2LHet","2RHet","3L","3R","3LHet","3RHet","4","X","XHet","YHet","U","Uextra","mito"); }
elsif ($organism eq "Yeast")
	{ $num_chroms = 16;
	  @chrom_names = ("chr01","chr02","chr03","chr04","chr05","chr06","chr07","chr08","chr09","chr10","chr11","chr12","chr13","chr14","chr15","chr16"); }
elsif ($organism eq "Test Genome")
	{ $num_chroms = 1; 
	  @chrom_names = ("chr01"); }
else
	{ $answer = $mw -> messageBox(-message=>"Invalid organism",-type=>'Ok'); 
	  return; }


$chrom_count = 0;
$exon_count = 0;
while ($chrom_count < $num_chroms)
{
$chrom_name = $chrom_names[$chrom_count];

if ($samptype eq "All")
	{
	$query1 = "select chromosome,start_pos,end_pos,number,side from $short_read_table where (chromosome = \"$chrom_name\") order by start_pos ASC";
	}
else
	{
	$query1 = "select chromosome,start_pos,end_pos,number,side from $short_read_table where (sample_type = \"$samptype\") and (chromosome = \"$chrom_name\") order by start_pos ASC";
	}


$sqlquery1 = $dbh->prepare($query1) or die "Cannot prepare query \n";
$rv1 = $sqlquery1->execute or die "Cannot execute query \n";
@row = $sqlquery1->fetchrow_array();

# set the exon start and end to the first read
$chrom = $row[0];
$exon_start = $row[1];
$exon_end = $row[2];
$exon_num = $row[3];
$exon_side = $row[4];
$read_count = 1;
if ($exon_side eq "L")
	{ $l_count = 1;
	  $r_count = 0; }
else
	{ $l_count = 0;
	  $r_count = 1; }
$num_count = $exon_num;

while (defined $row[0])
{
	@row = $sqlquery1->fetchrow_array();
	$read_start = $row[1];
	$read_end = $row[2];
	$read_num = $row[3];
	$read_side = $row[4];

	while ((defined $row[0]) and ($read_start <= $exon_end))
	{	
		if ($read_side eq "L")
			{ $l_count++; }
		else
			{ $r_count++; }
		$num_count = $num_count + $read_num;


		$read_count++;
		if ($read_end > $exon_end)
			{ $exon_end = $read_end; }

		@row = $sqlquery1->fetchrow_array();
		$read_start = $row[1];
		$read_end = $row[2];
		$read_num = $row[3];
		$read_side = $row[4];
	}


	$exon_length = $exon_end - $exon_start;

#	if (($num_count >= $min_reads) and ($l_count > 0) and ($r_count > 0))
	if ($num_count >= $min_reads)
	{

		# print out all the exons up til the exon found by the junction alignments
		while (($end_pos1 < $exon_start) and ($chrom1 eq $chrom))
			{
			$debug++;
			print OUT_FILE "$line";

			$line = <IN_FILE>;
			@row1 = split("\t",$line);
			$chrom1 = $row1[0];
			$start_pos1 = $row1[1];
			$end_pos1 = $row1[2];
			$exon_len1 = $row1[3];
			$read_count1 = $row1[4];
			$read_count1 =~ s/\s+$//; #remove trailing spaces
			}

		# At this point, $end_pos1 (the end of the exon in the text file) >= $exon_start (the start of the exon identified by the junctions)
		if ($start_pos1 > $exon_end) # the identified exon is not in the input file
			{ print OUT_FILE "$chrom\t$exon_start\t$exon_end\t$exon_length\t$num_count\n"; }
		elsif (($start_pos1 <= $exon_start) && ($end_pos1 >= $exon_end)) # the junc exon is contained in the exon in the input file
			{ print OUT_FILE "$line"; 
			  # skip junc exons that are contained in the exon in the input file
			  while ((defined $row[0]) and ($read_start <= $end_pos1))
				{	
				@row = $sqlquery1->fetchrow_array();
				$read_start = $row[1];
				}
			  &read_next_line();
			}
		elsif (($start_pos1 >= $exon_start) && ($end_pos1 >= $exon_end)) # the junc exon overlaps with an exon in the input file
			{ $exon_length = $end_pos1 - $exon_start;
			  print OUT_FILE "$chrom\t$exon_start\t$end_pos1\t$exon_length\t$read_count1\n";
			  # skip junc exons that are contained in the exon in the input file
			  while ((defined $row[0]) and ($read_start <= $end_pos1))
				{	
				@row = $sqlquery1->fetchrow_array();
				$read_start = $row[1];
				}
			  &read_next_line();
			}
		elsif (($start_pos1 >= $exon_start) && ($end_pos1 <= $exon_end)) # the exon in the file is contained within the junc exon
			{ # I assume that there is at most one exon contained in the junc exon
			  $temp_count = $read_count1;
			  &read_next_line();
			  if ($start_pos1 <= $exon_end)
				{ $exon_length = $end_pos1 - $exon_start;
				  $temp_count = $temp_count + $read_count1;
				  print OUT_FILE "$chrom\t$exon_start\t$end_pos1\t$exon_length\t$temp_count\n"; 
				  while ((defined $row[0]) and ($read_start <= $end_pos1))
					{	
					@row = $sqlquery1->fetchrow_array();
					$read_start = $row[1];
					}
				  &read_next_line();
				}
			  else
				{ print OUT_FILE "$chrom\t$exon_start\t$exon_end\t$exon_length\t$temp_count\n"; }
			}
		elsif (($start_pos1 <= $exon_start) && ($end_pos1 <= $exon_end)) # the junc exon overlaps with the exon in the input file
			{
			  $exon_start = $start_pos1;
			  $temp_count = $read_count1;
			  &read_next_line();
			  if ($start_pos1 <= $exon_end)
				{ $exon_length = $end_pos1 - $exon_start;
				  $temp_count = $temp_count + $read_count1;
				  print OUT_FILE "$chrom\t$exon_start\t$end_pos1\t$exon_length\t$temp_count\n"; 
				  while ((defined $row[0]) and ($read_start <= $end_pos1))
					{	
					@row = $sqlquery1->fetchrow_array();
					$read_start = $row[1];
					}
				  &read_next_line();
				}
			  else
				{ $exon_length = $exon_end - $exon_start;
				  print OUT_FILE "$chrom\t$exon_start\t$exon_end\t$exon_length\t$temp_count\n"; }
			}

		$exon_count++;
		if (($exon_count % 1000) == 0)
		{
			print "$exon_count\t$chrom\t$exon_start\t$exon_end\t$exon_length\t$read_count\t$num_count\n";
		}
	} # endif

	# reset the exon_start, exon_end to the next junction
	$exon_start = $row[1];
	$exon_end = $row[2];
	$exon_num = $row[3];
	$exon_side = $row[4];
	$read_count = 1;
	if ($exon_side eq "L")
		{ $l_count = 1;
		  $r_count = 0; }
	else
		{ $l_count = 0;
		  $r_count = 1; }
	$num_count = $exon_num;

} #endwhile

$debug++;
# print out the rest of the lines of IN_FILE that belong to the chromosome
while ($chrom1 eq $chrom_names[$chrom_count])
	{
	print OUT_FILE "$line";

	$line = <IN_FILE>;
	@row = split("\t",$line);
	$chrom1 = $row[0];
	$start_pos1 = $row[1];
	$end_pos1 = $row[2];
	$exon_len1 = $row[3];
	$read_count1 = $row[4];
	$read_count1 =~ s/\s+$//; #remove trailing spaces
	}



$chrom_count++;
}

close(OUT_FILE);
close(IN_FILE);

print "Finished\n";
}

#############################################################################################################################################

sub read_next_line
{
	$line = <IN_FILE>;
	@row1 = split("\t",$line);
	$chrom1 = $row1[0];
	$start_pos1 = $row1[1];
	$end_pos1 = $row1[2];
	$exon_len1 = $row1[3];
	$read_count1 = $row1[4];
	$read_count1 =~ s/\s+$//; #remove trailing spaces

	# if we reached the end of the file, set $start_pos1 to a really large number so that any following exons from junctions are printed.
	if (not defined $row1[0])
		{ $start_pos1 = 10000000000;}
}

