#!/usr/bin/perl

# link_exons - This program links potential exons together that are within a specified distance to avoid false gaps due to low read coverage
# Written by Kenneth Watanabe, University of Nevada Las Vegas
# Copyright (c) 2014

use Tk;
use Tk::ROText;
use Tk::BrowseEntry;
use File::Basename;

# The following code creates a file to store the previous values
($name,$path,$suffix)=fileparse($0,"\.[^.]*");
$default_file = $ENV{'HOME'} . "/perl/".$name.".default";
open(DEFAULT_VALS,$default_file);

$mw = MainWindow->new(-title => $name);
# In the geometry function, the 800x400 is the size of the window 800 pixels horizontal, 300 pixels vertical
# +400+200 is the position on the screen where the window is to be pasted, 400 pixels from the left of the screen, 200 pixels from the top
$mw -> geometry('800x400+400+200');

# Create frame f1 in the main window 
$f1 = $mw->Frame(-relief => 'ridge', -borderwidth => 2) ->pack(-fill => 'x');
$f2 = $mw->Frame(-relief => 'ridge', -borderwidth => 2) ->pack(-fill => 'x');
$f3 = $mw->Frame(-relief => 'ridge', -borderwidth => 2) ->pack(-fill => 'x');
$f4 = $mw->Frame(-relief => 'ridge', -borderwidth => 2) ->pack(-fill => 'x');
$f5 = $mw->Frame(-relief => 'ridge', -borderwidth => 2) ->pack(-fill => 'x');
$f6 = $mw->Frame(-relief => 'ridge', -borderwidth => 2) ->pack(-fill => 'x');
$f7 = $mw->Frame(-relief => 'ridge', -borderwidth => 2) ->pack(-fill => 'x');
$fbot = $mw->Frame() ->pack(-side=>'bottom',-fill => 'x');

# Create an go button
$go = $fbot->Button(-text => 'Go', -command => \&do_search)
    ->pack(-side => 'left');
# Create an exit button
$ex = $fbot->Button(-text => 'Exit', -command => \&do_exit)
    ->pack(-side => 'right');
# Create an edit button
$ed = $fbot->Button(-text => 'View Results', -command => \&do_edit)
    ->pack();


# Creates Browse Entry pulldown box in frame f1
$filename1 = <DEFAULT_VALS>;
$filename1 =~ s/\s+$//; #remove trailing spaces
if ($filename1 ne "") { ($name,$path,$suffix)=fileparse($filename1,"\.[^.]*"); }
$f1 -> Label(-text=>"Input Exon File: ") -> pack(-side => 'left');  # creates a label in frame $f
$be1 = $f1->BrowseEntry(-variable => \$filename1,-width=>50)->pack(-side => 'left');
# If user types in word and hits return, invoke do_search
$be1->bind("<Return>", \&do_search);
$be1->focus;  # Start w/focus on BrowseEntry
$f1 -> Button(-text=>'Search',-command =>\&get_file1) -> pack(-side=>'left');

$filename2 = <DEFAULT_VALS>;
$filename2 =~ s/\s+$//; #remove trailing spaces
if ($filename2 ne "") { ($name,$path,$suffix)=fileparse($filename1,"\.[^.]*"); }
$f2 -> Label(-text=>"Output Exon File: ") -> pack(-side => 'left');  # creates a label in frame $f
$be2 = $f2->BrowseEntry(-variable => \$filename2,-width=>50)->pack(-side => 'left');
$f2 -> Button(-text=>'Search',-command =>\&get_file2) -> pack(-side=>'left');

$threshold = <DEFAULT_VALS>;
$threshold =~ s/\s+$//; #remove trailing spaces
$f3 -> Label(-text=>"Threshold (nt): ") -> pack(-side => 'left');  # creates a label in frame $f
$be3 = $f3->BrowseEntry(-variable => \$threshold,-width=>50)->pack(-side => 'left');

$organism = <DEFAULT_VALS>;
$organism =~ s/\s+$//; #remove trailing spaces
$f4 -> Label(-text=>"Organism: ") -> pack(-side => 'left');  # creates a label in frame $f
$be4 = $f4->BrowseEntry(-variable => \$organism,-width=>50)->pack(-side => 'left');

# Initialize browse entry with some default values
$be4->insert('end', "Oryza sativa");
$be4->insert('end', "Arabidopsis thaliana");
$be4->insert('end', "Caenorhabditis elegans");
$be4->insert('end', "Drosophila melanogaster");
$be4->insert('end', "Yeast");
$be4->insert('end', "Test Genome");

# Initialize browse entry with some default values
opendir ( FDIR , "." )  || die "Can not open current directory.\n";
foreach $filename (sort readdir(FDIR)) {
	next if ( $filename eq "." || $filename eq ".." );
	$be1->insert('end', $filename);
}

close(DEFAULT_VALS);

MainLoop;

###############################################################################################
# This routine is called when the "Search" button is clicked

sub get_file1 {
$filename1 = $mw -> getOpenFile(-initialdir=>$path);
}

sub get_file2 {
$filename2 = $mw -> getOpenFile(-initialdir=>$path);
}

sub get_file3 {
$filename3 = $mw -> getOpenFile(-initialdir=>$path);
}

sub get_file4 {
$filename4 = $mw -> getOpenFile(-initialdir=>$path);
}
###############################################################################################

sub do_exit {
# Put commands here to be executed on exit
  exit;
}

sub save_defaults {
open(DEFAULT_VALUES,">$default_file");
$filename1 =~ s/\s+$//; #remove trailing spaces
$filename2 =~ s/\s+$//; #remove trailing spaces
$threshold =~ s/\s+$//; #remove trailing spaces
$organism =~ s/\s+$//; #remove trailing spaces
print DEFAULT_VALUES "$filename1\n";
print DEFAULT_VALUES "$filename2\n";
print DEFAULT_VALUES "$threshold\n";
print DEFAULT_VALUES "$organism\n";
close(DEFAULT_VALUES);
}

###############################################################################################

sub do_edit {
$edit_cmd = "gedit " . $filename2;
system $edit_cmd;
}

###############################################################################################

sub do_search
{
&save_defaults;

open(EXON_FILE,$filename1) || die "Cannot open input file $filename2\n";
open(OUT_EXON,">$filename2") || die "Cannot open input file $filename4\n";

&init_arrays();

$debug = 0;

$header_line = <EXON_FILE>;
print OUT_EXON "$header_line";

$last_line = <EXON_FILE>;
@row = split ("\t",$last_line);
$last_chrom = $row[0];
$last_start = $row[1];
$last_end = $row[2];
$last_length = $row[3];
$last_score = $row[4];
$last_score =~ s/\s+$//; #remove trailing spaces

$exon_count = 1;
$line_count = 0;
while (<EXON_FILE>)
{
	$line_count++;
	if (($line_count % 100) == 0)
		{ print "$line_count\t$_"; }
	@row = split ("\t",$_);
	$chrom = $row[0];
	$start_pos = $row[1];
	$end_pos = $row[2];
	$length = $row[3];
	$score = $row[4];
	$score =~ s/\s+$//; #remove trailing spaces
	$dist_to_next = $start_pos - $last_end;

	if ((($dist_to_next > $threshold) and ($chrom eq $last_chrom)) or ($chrom ne $last_chrom))
	{
		print OUT_EXON "$last_chrom\t$last_start\t$last_end\t$last_length\t$last_score\n";

		#reset start position
		$last_length = $length;
		$last_score = $score;
		$last_start = $start_pos;
	}
	else
	{
		$debug++;
		# combine the last exon with the next exon
		$last_length = $end_pos - $last_start + 1;
		$last_score = $last_score + $score;
		$last_start = $last_start;	# dont change start pos
	}
	$last_chrom = $chrom;
	$last_end = $end_pos;

}
# print last exon
print OUT_EXON "$last_chrom\t$last_start\t$last_end\t$last_length\t$last_score\n";

close(EXON_FILE);
close(OUT_EXON);

print "Finished\n";
}

########################################################################################################################

sub init_arrays
{
if ($organism eq "Oryza sativa")
	{ $num_chroms = 12;
	  %chrom_names = ("chr01","01","chr02","02","chr03","03","chr04","04",
			"chr05","05","chr06","06","chr07","07","chr08","08",
			"chr09","09","chr10","10","chr11","11","chr12","12"); }
elsif ($organism eq "Arabidopsis thaliana")
	{ $num_chroms = 5; 
	  %chrom_names = ("chr1","01","chr2","02","chr3","03","chr4","04","chr5","05"); }
elsif ($organism eq "Caenorhabditis elegans")
	{ $num_chroms = 7; 
	  %chrom_names = ("chr1","01","chr2","02","chr3","03","chr4","04","chr5","05","chrX","X","chrM","M"); }
elsif ($organism eq "Drosophila melanogaster")
	{ $num_chroms = 15; 
	  %chrom_names = ("2L","2R","2LHet","2RHet","3L","3R","3LHet","3RHet","4","X","XHet","YHet","U","Uextra","mito"); }
elsif ($organism eq "Saccharmyces cerevisiae")
	{ $num_chroms = 16;
	  %chrom_names = ("chr01","01","chr02","02","chr03","03","chr04","04",
			"chr05","05","chr06","06","chr07","07","chr08","08",
			"chr09","09","chr10","10","chr11","11","chr12","12",
			"chr03","03","chr14","14","chr15","15","chr16","16"); }
elsif ($organism eq "Test Genome")
	{ $num_chroms = 1; 
	  %chrom_names = ("chr01","01"); }
}
