#!/usr/bin/perl

# load_junctions - Load junction and junction_ends table from a junctions.bed file generated by Tophat. The junction and junction_ends table must be created prior to running this program.
# Written by Kenneth Watanabe, University of Nevada Las Vegas
# Copyright (c) 2014

use Tk;
use Tk::ROText;
use Tk::BrowseEntry;
use File::Basename;
use DBI;			# this give access to SQL database commands

require "connectDB.pl";

our $uname;
our $passwd;

# The following code creates a file to store the previous values
($name,$path,$suffix)=fileparse($0,"\.[^.]*");
$default_file = $ENV{'HOME'} . "/perl/".$name.".default";
open(DEFAULT_VALS,$default_file);

$mw = MainWindow->new(-title => $name);
# In the geometry function, the 800x400 is the size of the window 600 pixels horizontal, 400 pixels vertical
# +400+200 is the position on the screen where the window is to be pasted, 400 pixels from the left of the screen, 200 pixels from the top 
$mw -> geometry('800x400+400+200');

# Create frame f1 in the main window 
$f1 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f2 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f3 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f4 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f5 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f6 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f7 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');

# Create an go button
$go = $mw->Button(-text => 'Go', -command => \&do_search)
    ->pack(-side => 'left');
# Create an exit button
$ex = $mw->Button(-text => 'Exit', -command => \&do_exit)
    ->pack(-side => 'right');

# Creates Browse Entry pulldown box in frame f1
$filename1 = <DEFAULT_VALS>;
$filename1 =~ s/\s+$//; #remove trailing spaces
if ($filename1 ne "") { ($name,$path,$suffix)=fileparse($filename1,"\.[^.]*"); }
$f1 -> Label(-text=>"File name: ") -> pack(-side => 'left');  # creates a label in frame $f
$be1 = $f1->BrowseEntry(-variable => \$filename1,-width=>50)->pack(-side => 'left');
# If user types in word and hits return, invoke do_search
$be1->bind("<Return>", \&do_search);
$be1->focus;  # Start w/focus on BrowseEntry

$f1 -> Button(-text=>'Search',-command =>\&get_file1) -> pack(-side=>'left');


$dbase_name = <DEFAULT_VALS>;
$dbase_name =~ s/\s+$//; #remove trailing spaces
$f2 -> Label(-text=>"Database name: ") -> pack(-side => 'left');  # creates a label in frame $f
$be2 = $f2->BrowseEntry(-variable => \$dbase_name,-width=>50)->pack(-side => 'left');


$table_name = <DEFAULT_VALS>;
$table_name =~ s/\s+$//; #remove trailing spaces
$f3 -> Label(-text=>"Table name 1: ") -> pack(-side => 'left');  # creates a label in frame $f
$be3 = $f3->BrowseEntry(-variable => \$table_name,-width=>50)->pack(-side => 'left');


$table_name2 = <DEFAULT_VALS>;
$table_name2 =~ s/\s+$//; #remove trailing spaces
$f4 -> Label(-text=>"Table name 2: ") -> pack(-side => 'left');  # creates a label in frame $f
$be4 = $f4->BrowseEntry(-variable => \$table_name2,-width=>50)->pack(-side => 'left');


# Initialize browse entry with some default values
$host = "localhost";
$dbase_name = "deep_seq";
$dbh = DBI -> connect("DBI:mysql:information_schema:$host",$uname,$passwd);
$query1 = "select table_name from tables where table_schema = \"$dbase_name\" order by table_name ASC";
$sqlquery1 = $dbh->prepare($query1) or die "Cannot prepare query \n";
$rv1 = $sqlquery1->execute or die "Cannot execute query \n";
@row = $sqlquery1->fetchrow_array();
while (defined $row[0])
{
	$be3->insert('end', $row[0]);
	$be4->insert('end', $row[0]);
	@row = $sqlquery1->fetchrow_array();
}
$sqlquery1->finish;


$delimiter = <DEFAULT_VALS>;
$delimiter =~ s/\s+$//; #remove trailing spaces
$f5 -> Label(-text=>"Delimiter: ") -> pack(-side => 'left');  # creates a label in frame $f
$be5 = $f5->BrowseEntry(-variable => \$delimiter,-width=>50)->pack(-side => 'left');
$be5->insert('end', "Comma");
$be5->insert('end', "Tab");

$samp_name = <DEFAULT_VALS>;
$samp_name =~ s/\s+$//; #remove trailing spaces
$f6 -> Label(-text=>"Sample Name: ") -> pack(-side => 'left');  # creates a label in frame $f
$be6 = $f6->BrowseEntry(-variable => \$samp_name,-width=>50)->pack(-side => 'left');

$samp_type = <DEFAULT_VALS>;
$samp_type =~ s/\s+$//; #remove trailing spaces
$f7 -> Label(-text=>"Sample Type: ") -> pack(-side => 'left');  # creates a label in frame $f
$be7 = $f7->BrowseEntry(-variable => \$samp_type,-width=>50)->pack(-side => 'left');

$be7->insert('end', "control");
$be7->insert('end', "ABA");
$be7->insert('end', "GA");
$be7->insert('end', "GA-ABA");

close(DEFAULT_VALS);

MainLoop;

###############################################################################################
# This routine is called when the "Search" button is clicked

sub get_file1 {
$filename1 = $mw -> getOpenFile(-initialdir=>$path);
if ($filename1 ne "") { ($name,$path,$suffix)=fileparse($filename1,"\.[^.]*"); }
}

###############################################################################################

sub do_exit {
# Put commands here to be executed on exit
  exit;
}

sub save_defaults {
open(DEFAULT_VALUES,">$default_file");
$filename1 =~ s/\s+$//; #remove trailing spaces
$dbase_name =~ s/\s+$//; #remove trailing spaces
$table_name =~ s/\s+$//; #remove trailing spaces
$table_name2 =~ s/\s+$//; #remove trailing spaces
$delimiter =~ s/\s+$//; #remove trailing spaces
$samp_name =~ s/\s+$//; #remove trailing spaces
$samp_type =~ s/\s+$//; #remove trailing spaces
print DEFAULT_VALUES "$filename1\n";
print DEFAULT_VALUES "$dbase_name\n";
print DEFAULT_VALUES "$table_name\n";
print DEFAULT_VALUES "$table_name2\n";
print DEFAULT_VALUES "$delimiter\n";
print DEFAULT_VALUES "$samp_name\n";
print DEFAULT_VALUES "$samp_type\n";
close(DEFAULT_VALUES);
}

###############################################################################################

sub do_search
{
&save_defaults;
&init_arrays;

$host = "localhost";
$database = $dbase_name;

$dbh = DBI -> connect("DBI:mysql:$database:$host",$uname,$passwd);
$debug++;

if ($delimiter eq "Comma")
	{ $dl = ","; }
else
	{ $dl = "\t"; }

open(INFILE,$filename1);
$header_line = <INFILE>;	# skip first line

# set up query for junctions table
$init_query = "insert into ".$table_name." (".$fields[0];
$i = 1;
while ($i < $num_fields)
{
	$fields[$i] =~ s/\s+$//; 	#remove trailing spaces
	$init_query =$init_query.",".$fields[$i];
	$i++;
}
$init_query =$init_query.") VALUES (";

# set up query for junction_ends table
$init_query2 = "insert into ".$table_name2." (".$fields2[0];
$i = 1;
while ($i < $num_fields2)
{
	$fields2[$i] =~ s/\s+$//; 	#remove trailing spaces
	$init_query2 =$init_query2.",".$fields2[$i];
	$i++;
}
$init_query2 =$init_query2.") VALUES (";


$line_count = 0;
while (<INFILE>)
{
	$line_count++;
	$line = $_;
	if (($line_count%1000) == 0)
		{ print "$line_count \t $line"; }
	@lines = split($dl,$line);
	($left_side,$right_side) = split(",",$lines[10]);

	$values[0] = $lines[0];		# chromosome
	$values[1] = $lines[1];		# start_pos
	$values[2] = $lines[2];		# end_pos
	$values[3] = $lines[3];		# junction_name
	$values[4] = $lines[4];		# number
	$values[5] = $lines[5];		# strand
	$values[6] = $samp_type;	# sample_type
	$values[7] = $samp_name;	# sample_name
	$values[8] = $left_side;	# left_side
	$values[9] = $right_side;	# right_side
	$values[10] = $lines[1] + $left_side;	# intron start 
	$values[11] = $lines[2] - $right_side;	# intron end


# insert junction
	$query = $init_query."\"".$values[0]."\"";
	$i = 1;
	while ($i < $num_fields)
	{
		$temp_data = $values[$i];
		$temp_data =~ s/\s+$//; 	#remove trailing spaces
		$query =$query.",\"".$temp_data."\"";
		$i++;
	}
	$query =$query.")";
	$debug++;

	$sqlquery = $dbh->prepare($query) or die "Cannot prepare query \n";
	$rv = $sqlquery-> execute or die "Cannot execute query \n";



# Insert left side of junction
	$values[2] = $lines[1] + $left_side;	# end_pos
	$values[8] = "L";			# left_side
	$query = $init_query2."\"".$values[0]."\"";
	$i = 1;
	while ($i < $num_fields2)
	{
		$temp_data = $values[$i];
		$temp_data =~ s/\s+$//; 	#remove trailing spaces
		$query =$query.",\"".$temp_data."\"";
		$i++;
	}
	$query =$query.")";
	$debug++;

	$sqlquery = $dbh->prepare($query) or die "Cannot prepare query \n";
	$rv = $sqlquery-> execute or die "Cannot execute query \n";

# Insert right side of junction
	$values[1] = $lines[2] - $right_side;	# start_pos
	$values[2] = $lines[2];			# end_pos
	$values[8] = "R";			# right_side
	$query = $init_query2."\"".$values[0]."\"";
	$i = 1;
	while ($i < $num_fields2)
	{
		$temp_data = $values[$i];
		$temp_data =~ s/\s+$//; 	#remove trailing spaces
		$query =$query.",\"".$temp_data."\"";
		$i++;
	}
	$query =$query.")";
	$debug++;

	$sqlquery = $dbh->prepare($query) or die "Cannot prepare query \n";
	$rv = $sqlquery-> execute or die "Cannot execute query \n";


}

$rc = $sqlquery->finish;


print "Finished\n";
}



sub init_arrays
{

@fields =  ("chromosome","start_pos","end_pos","junction_name","number","strand","sample_type","sample_name","left_side","right_side","intron_start","intron_end");
$num_fields = scalar(@fields);


@fields2 =  ("chromosome","start_pos","end_pos","junction_name","number","strand","sample_type","sample_name","side");
$num_fields2 = scalar(@fields2);


} # sub init_arrays

