#!/usr/bin/perl

# load_sam_file - This program loads a sam file into a MySQL database table. The table must be created before running the program with the column names specified in the header file.
# Written by Kenneth Watanabe, University of Nevada Las Vegas
# Copyright (c) 2014


use Tk;
use Tk::ROText;
use Tk::BrowseEntry;
use File::Basename;
use DBI;			# this give access to SQL database commands

require "connectDB.pl";

our $uname;
our $passwd;

# Load previous values from file
($name,$path,$suffix)=fileparse($0,"\.[^.]*");
$default_file = $ENV{'HOME'} . "/perl/".$name.".default";
open(DEFAULT_VALS,$default_file);

$mw = MainWindow->new(-title => $name);
# In the geometry function, the 800x300 is the size of the window 800 pixels horizontal, 300 pixels vertical
# +400+200 is the position on the screen where the window is to be pasted, 400 pixels from the left of the screen, 200 pixels from the top 
$mw -> geometry('800x400+400+200');

# Create frames in the main window 
$f1 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)->pack(-fill => 'x');
$f2 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)->pack(-fill => 'x');
$f3 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)->pack(-fill => 'x');
$f4 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)->pack(-fill => 'x');
$f5 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)->pack(-fill => 'x');
$f6 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)->pack(-fill => 'x');
$f7 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)->pack(-fill => 'x');
$f8 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)->pack(-fill => 'x');
$fbot = $mw->Frame() ->pack(-side=>'bottom',-fill => 'x');

# Create an go button
$go = $fbot->Button(-text => 'Go', -command => \&do_search)->pack(-side => 'left');
# Create an exit button
$ex = $fbot->Button(-text => 'Exit', -command => \&do_exit)->pack(-side => 'right');


# Creates Browse Entry pulldown box in frame f1
$filename1 = <DEFAULT_VALS>;
$filename1 =~ s/\s+$//; #remove trailing spaces
if ($filename1 ne "") { ($name,$path,$suffix)=fileparse($filename1,"\.[^.]*"); }
$f1 -> Label(-text=>"File name: ") -> pack(-side => 'left');  # creates a label in frame $f
$be1 = $f1->BrowseEntry(-variable => \$filename1,-width=>50)->pack(-side => 'left');
# If user types in word and hits return, invoke do_search
$be1->bind("<Return>", \&do_search);
$be1->focus;  # Start w/focus on BrowseEntry

$f1 -> Button(-text=>'Search',-command =>\&get_file1) -> pack(-side=>'left');


$dbase_name = <DEFAULT_VALS>;
$dbase_name =~ s/\s+$//;
$f2 -> Label(-text=>"Database name: ") -> pack(-side => 'left'); 
$be2 = $f2->BrowseEntry(-variable => \$dbase_name,-width=>50)->pack(-side => 'left');


$table_name = <DEFAULT_VALS>;
$table_name =~ s/\s+$//;
$f3 -> Label(-text=>"Table name: ") -> pack(-side => 'left');  
$be3 = $f3->BrowseEntry(-variable => \$table_name,-width=>50)->pack(-side => 'left');

# Initialize browse entry with some default values
$host = "localhost";
$dbh = DBI -> connect("DBI:mysql:information_schema:$host",$uname,$passwd);
$query1 = "select table_name from tables where table_schema = \"$dbase_name\" order by table_name ASC";
$sqlquery1 = $dbh->prepare($query1) or die "Cannot prepare query \n";
$rv1 = $sqlquery1->execute or die "Cannot execute query \n";
@row = $sqlquery1->fetchrow_array();
while (defined $row[0])
{
	$be3->insert('end', $row[0]);
	@row = $sqlquery1->fetchrow_array();
}
$sqlquery1->finish;


$samp_type = <DEFAULT_VALS>;
$samp_type =~ s/\s+$//; 
$f4 -> Label(-text=>"Sample Type: ") -> pack(-side => 'left'); 
$be4 = $f4->BrowseEntry(-variable => \$samp_type,-width=>50)->pack(-side => 'left');
$be4->insert('end', "control");
$be4->insert('end', "ABA");
$be4->insert('end', "GA");
$be4->insert('end', "GA-ABA");


$samp_name = <DEFAULT_VALS>;
$samp_name =~ s/\s+$//; 
$f5 -> Label(-text=>"Sample Name: ") -> pack(-side => 'left');
$be5 = $f5->BrowseEntry(-variable => \$samp_name,-width=>50)->pack(-side => 'left');


$delimiter = <DEFAULT_VALS>;
$delimiter =~ s/\s+$//;
$f6 -> Label(-text=>"Delimiter: ") -> pack(-side => 'left'); 
$be6 = $f6->BrowseEntry(-variable => \$delimiter,-width=>50)->pack(-side => 'left');
$be6->insert('end', "Comma");
$be6->insert('end', "Tab");

$header_file = <DEFAULT_VALS>;
$header_file =~ s/\s+$//; 
$f7 -> Label(-text=>"Header File: ") -> pack(-side => 'left');
$be7 = $f7->BrowseEntry(-variable => \$header_file,-width=>50)->pack(-side => 'left');
$f7 -> Button(-text=>'Search',-command =>\&get_file2) -> pack(-side=>'left');

$save_type = "Y";
$f8 -> Label(-text=>"Update Sample Type/Name: ") -> pack(-side => 'left');
$f8->Radiobutton(-variable=>\$save_type,-text=>'Yes',-value=>'Y') -> pack(-side=>'left');
$f8->Radiobutton(-variable=>\$save_type,-text=>'No',-value=>'N') -> pack(-side=>'left');


# Initialize browse entry with some default values
opendir ( FDIR , "." )  || die "Can not open current directory.\n";
foreach $filename (sort readdir(FDIR)) {
	next if ( $filename eq "." || $filename eq ".." );
	$be1->insert('end', $filename);
}

close(DEFAULT_VALS);

MainLoop;

###############################################################################################
# This routine is called when the "Search" button is clicked

sub get_file1 {
$filename1 = $mw -> getOpenFile(-initialdir=>$path);
}

sub get_file2 {
$header_file = $mw -> getOpenFile(-initialdir=>$path);
}

###############################################################################################

sub do_exit {
# Put commands here to be executed on exit
  exit;
}

sub save_defaults {
open(DEFAULT_VALUES,">$default_file");
$filename1 =~ s/\s+$//; #remove trailing spaces
$dbase_name =~ s/\s+$//; #remove trailing spaces
$table_name =~ s/\s+$//; #remove trailing spaces
$samp_type =~ s/\s+$//; #remove trailing spaces
$samp_name =~ s/\s+$//; #remove trailing spaces
$delimiter =~ s/\s+$//; #remove trailing spaces
$header_file =~ s/\s+$//; #remove trailing spaces
print DEFAULT_VALUES "$filename1\n";
print DEFAULT_VALUES "$dbase_name\n";
print DEFAULT_VALUES "$table_name\n";
print DEFAULT_VALUES "$samp_type\n";
print DEFAULT_VALUES "$samp_name\n";
print DEFAULT_VALUES "$delimiter\n";
print DEFAULT_VALUES "$header_file\n";
close(DEFAULT_VALUES);
}

###############################################################################################

sub do_search
{
&save_defaults;

$host = "localhost";
$database = $dbase_name;

$dbh = DBI -> connect("DBI:mysql:$database:$host",$uname,$pass);
$debug++;

if ($delimiter eq "Comma")
	{ $dl = ","; }
else
	{ $dl = "\t"; }

# Load the field names into the array @fields
open(HEADERFILE,$header_file);
$line = <HEADERFILE>;
close (HEADERFILE);
@fields = split($dl,$line);
$num_fields = scalar(@fields);

$init_query = "insert into ".$table_name." (".$fields[0];
$i = 1;
while ($i < $num_fields)
{
	$temp_field = $fields[$i];
	$temp_field =~ s/\s+$//; 	#remove trailing spaces
	$fields[$i] = $temp_field;
	$init_query =$init_query.",".$temp_field;
	$i++;
}
$debug++;
if ($save_type eq "Y")
	{ $init_query =$init_query.",sample_name,sample_type,end_position) VALUES ("; }
else
	{ $init_query =$init_query.") VALUES ("; }


open(INFILE,$filename1);
$line_count = 0;
while (<INFILE>)
{
	$line_count++;
	$line = $_;

	if (($line_count%1000) == 0)
		{ print "$line_count \t $line"; }
	@lines = split($dl,$line);

	$query = $init_query."\"".$lines[0]."\"";
	$i = 1;
	while ($i < $num_fields)
	{	# cannot have fields called source, score or frame; for phred quality score, must replace slashes (\) since mysql doesn't like it
		if ($fields[$i] eq "quality")
			{
			$temp_data = $lines[$i];
			$temp_data =~ s/\s+$//; 	#remove trailing spaces
			$temp_data =~ s/\\/[/g; 	#replace slash (\) with a bracket ([)
			$temp_data =~ s/\"/!/g; 	#replace double quote (") with an exclamation point (!)
			$query =$query.",\"".$temp_data."\"";
			}
		else
			{
			$temp_data = $lines[$i];
			$temp_data =~ s/\s+$//; 	#remove trailing spaces
			$query =$query.",\"".$temp_data."\"";
			}
		$i++;
	}

#	$debug++;
	$cigar_error = "F";
	$pre_intron = "T";
	$left_side = 0;
	$right_side = 0;
	$mcount = 0;
	$ncount = 0;
	$dcount = 0;
	$icount = 0;
	$start_pos = $lines[3];
	$cigar = $lines[5];
	while (length($cigar) > 0)
	{
		$mpos = index($cigar,"M"); if ($mpos < 0) { $mpos = 1000; }
		$npos = index($cigar,"N"); if ($npos < 0) { $npos = 1000; }
		$dpos = index($cigar,"D"); if ($dpos < 0) { $dpos = 1000; }
		$ipos = index($cigar,"I"); if ($ipos < 0) { $ipos = 1000; }

		if (($mpos < $npos) and ($mpos < $dpos) and ($mpos < $ipos))
 		{ $num = substr($cigar,0,$mpos);
		  $mcount = $mcount + $num;
		  $cigar = substr($cigar,$mpos+1,1000); 
		  if ($pre_intron eq "T") 
			{ $left_side = $left_side + $num; } 
		  else
			{ $right_side = $right_side + $num; } 
		}
		elsif (($npos < $mpos) and ($npos < $dpos) and ($npos < $ipos))
 		{ $num = substr($cigar,0,$npos);
		  $ncount = $ncount + $num;
		  $cigar = substr($cigar,$npos+1,1000); 
		  $pre_intron = "F"; }
		elsif (($dpos < $mpos) and ($dpos < $npos) and ($dpos < $ipos))
 		{ $num = substr($cigar,0,$dpos);
		  $dcount = $dcount + $num;
		  $cigar = substr($cigar,$dpos+1,1000);
		  if ($pre_intron eq "T") 
			{ $left_side = $left_side + $num; } 
		  else
			{ $right_side = $right_side + $num; } 
		}
		elsif (($ipos < $mpos) and ($ipos < $npos) and ($ipos < $dpos))
 		{ $num = substr($cigar,0,$ipos);
		  $icount = $icount + $num;
		  $cigar = substr($cigar,$ipos+1,1000);
		  if ($pre_intron eq "T") 
			{ $left_side = $left_side - $num; } 
		  else
			{ $right_side = $right_side - $num; } 
		}
		else
		{ print LOG_FILE "Error in CIGAR string: $_\n"; 
		  $cigar_error = "T"; $cigar = ""; }

		$cigar =~ s/\s+$//; #remove trailing spaces
	} #endwhile length(cigar) > 0
	$end_pos = $start_pos + $mcount + $ncount - $icount + $dcount - 1;

	if ($end_pos - $start_pos > 100000)
	{ print "Error - read length is greater than 100,000nt\n";
	  $debug++; }

	if ($save_type eq "Y")
	{ $query =$query.",\"$samp_name\",\"$samp_type\",$end_pos)"; }
	else
	{ $query =$query.")"; }

	$sqlquery = $dbh->prepare($query) or die "Cannot prepare query \n";
	$rv = $sqlquery-> execute or die "Cannot execute query \n";

}

$rc = $sqlquery->finish;

close (INFILE);

print "Finished\n";
}


