#!/usr/bin/perl

# scan_exons - This program searches for junctions contained within a potential exon. If the read depth on the junction is below the specified limit, then the junction is considered an intron
# Written by Kenneth Watanabe, University of Nevada Las Vegas
# Copyright (c) 2014

use Tk;
use Tk::ROText;
use Tk::BrowseEntry;
use File::Basename;
use DBI;

require "connectDB.pl";

our $uname;
our $passwd;

# The following code creates a file to store the previous values
($name,$path,$suffix)=fileparse($0,"\.[^.]*");
$default_file = $ENV{'HOME'} . "/perl/".$name.".default";
open(DEFAULT_VALS,$default_file);

$mw = MainWindow->new(-title => $name);
# In the geometry function, the 800x300 is the size of the window 800 pixels horizontal, 300 pixels vertical
# +400+200 is the position on the screen where the window is to be pasted, 400 pixels from the top of the screen, 200 pixels from the bottom 
$mw -> geometry('800x300+400+200');

# Create frame f1 in the main window 
$f1 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f2 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f3 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f4 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f5 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$f6 = $mw->Frame(-relief => 'ridge', -borderwidth => 2)
  ->pack(-fill => 'x');
$fbot = $mw->Frame() ->pack(-side=>'bottom',-fill => 'x');

# Create an go button
$go = $fbot->Button(-text => 'Go', -command => \&do_search)
    ->pack(-side => 'left');
# Create an exit button
$ex = $fbot->Button(-text => 'Exit', -command => \&do_exit)
    ->pack(-side => 'right');
# Create an edit button
$ed = $fbot->Button(-text => 'View Results', -command => \&do_edit)
    ->pack();


# Creates Browse Entry pulldown box in frame f1
$filename1 = <DEFAULT_VALS>;
$filename1 =~ s/\s+$//; #remove trailing spaces
if ($filename1 ne "") { ($name,$path,$suffix)=fileparse($filename1,"\.[^.]*"); }
$f1 -> Label(-text=>"File name: ") -> pack(-side => 'left');  # creates a label in frame $f
$be1 = $f1->BrowseEntry(-variable => \$filename1,-width=>50)->pack(-side => 'left');
# If user types in word and hits return, invoke do_search
$be1->bind("<Return>", \&do_search);
$be1->focus;  # Start w/focus on BrowseEntry

$f1 -> Button(-text=>'Search',-command =>\&get_file1) -> pack(-side=>'left');


$filename2 = <DEFAULT_VALS>;
$filename2 =~ s/\s+$//; #remove trailing spaces
$f2 -> Label(-text=>"Output file: ") -> pack(-side => 'left');  # creates a label in frame $f
$be2 = $f2->BrowseEntry(-variable => \$filename2,-width=>50)->pack(-side => 'left');

$f2 -> Button(-text=>'Search',-command =>\&get_file2) -> pack(-side=>'left');

$samptype = <DEFAULT_VALS>;
$samptype =~ s/\s+$//; #remove trailing spaces
my $lab = $f3 -> Label(-text=>"Sample Type: ") -> pack(-side => 'left');  # creates a label in frame $f
$be3 = $f3->BrowseEntry(-variable => \$samptype,-width=>50)->pack(-side => 'left');


$intron_threshold = <DEFAULT_VALS>;
$intron_threshold =~ s/\s+$//; #remove trailing spaces
$f4 -> Label(-text=>"Intron threshold (0-100%): ") -> pack(-side => 'left');  # creates a label in frame $f
$be4 = $f4->BrowseEntry(-variable => \$intron_threshold,-width=>50)->pack(-side => 'left');


$junction_table = <DEFAULT_VALS>;
$junction_table =~ s/\s+$//; #remove trailing spaces
$f5 -> Label(-text=>"Junction table: ") -> pack(-side => 'left');  # creates a label in frame $f
$be5 = $f5->BrowseEntry(-variable => \$junction_table,-width=>50)->pack(-side => 'left');

$short_read_table = <DEFAULT_VALS>;
$short_read_table =~ s/\s+$//; #remove trailing spaces
$f6 -> Label(-text=>"Short read table: ") -> pack(-side => 'left');  # creates a label in frame $f
$be6 = $f6->BrowseEntry(-variable => \$short_read_table,-width=>50)->pack(-side => 'left');

# Initialize browse entry with some default values
$host = "localhost";
$dbase_name = "deep_seq";
$dbh = DBI -> connect("DBI:mysql:information_schema:$host",$uname,$passwd);
$query1 = "select table_name from tables where table_schema = \"$dbase_name\" order by table_name ASC";
$sqlquery1 = $dbh->prepare($query1) or die "Cannot prepare query \n";
$rv1 = $sqlquery1->execute or die "Cannot execute query \n";
@row = $sqlquery1->fetchrow_array();
while (defined $row[0])
{
	$be5->insert('end', $row[0]);
	$be6->insert('end', $row[0]);
	@row = $sqlquery1->fetchrow_array();
}
$sqlquery1->finish;


# Initialize browse entry with some default values
$be3->insert('end', "control");
$be3->insert('end', "ABA");
$be3->insert('end', "GA");
$be3->insert('end', "ABA+GA");
$be3->insert('end', "All");

close(DEFAULT_VALS);

MainLoop;

###############################################################################################
# This routine is called when the "Search" button is clicked

sub get_file1 {
$filename1 = $mw -> getOpenFile(-initialdir=>$path);
}

sub get_file2 {
$filename2 = $mw -> getOpenFile(-initialdir=>$path);
}

###############################################################################################

sub do_exit {
# Put commands here to be executed on exit
  exit;
}

sub save_defaults {
open(DEFAULT_VALUES,">$default_file");
$filename1 =~ s/\s+$//; #remove trailing spaces
$filename2 =~ s/\s+$//; #remove trailing spaces
$samptype =~ s/\s+$//; #remove trailing spaces
$intron_threshold =~ s/\s+$//; #remove trailing spaces
$junction_table =~ s/\s+$//; #remove trailing spaces
$short_read_table =~ s/\s+$//; #remove trailing spaces
print DEFAULT_VALUES "$filename1\n";
print DEFAULT_VALUES "$filename2\n";
print DEFAULT_VALUES "$samptype\n";
print DEFAULT_VALUES "$intron_threshold\n";
print DEFAULT_VALUES "$junction_table\n";
print DEFAULT_VALUES "$short_read_table\n";
close(DEFAULT_VALUES);
}

###############################################################################################

sub do_edit {
$edit_cmd = "gedit " . $filename2;
system $edit_cmd;
}

###############################################################################################

sub do_search
{

&save_defaults;

$database = "deep_seq";
$host = "localhost";
$dbh = DBI -> connect("DBI:mysql:$database:$host",$uname,$passwd);


#only check exons that have a length greater than $min_exon
$min_exon = 250;	

open(INFILE,$filename1) || die "Cannot open input file $filename1\n";
open(OUTFILE,">$filename2") || die "Cannot open output file $filename2\n";
print OUTFILE "Chromosome\tStart Position\tEnd Position\tGene Length\tRead Count\n";


$header_line = <INFILE>;	# skip header

$line_count = 0;
while (<INFILE>)
{
	$line_count++;
	if (($line_count % 100) == 0)
	{
		print "$line_count\t$_";
	}

	@row = split("\t",$_);

	$chrom1 = $row[0];
	$start_pos1 = $row[1];
	$end_pos1 = $row[2];
	$exon_len1 = $row[3];
	$read_count1 = $row[4];
	$read_count1 =~ s/\s+$//; #remove trailing spaces

	#only check exons that have a length greater than $min_exon
	if ($exon_len1 < $min_exon)
	{
	print OUTFILE "$_";
	}

	else

	{

	# search for junctions within the exon
if ($samptype eq "All")
	{
	$query = "SELECT junction_name,number,start_pos,end_pos,chromosome,left_side,right_side FROM $junction_table WHERE (chromosome = \"$chrom1\") AND ((start_pos >= $start_pos1) AND (end_pos <= $end_pos1)) order by number desc";

	}
else
	{
	$query = "SELECT junction_name,number,start_pos,end_pos,chromosome,left_side,right_side FROM $junction_table WHERE (chromosome = \"$chrom1\") AND ((start_pos >= $start_pos1) AND (end_pos <= $end_pos1)) and (sample_type = \"$samptype\") order by number desc";
	}

	$sqlquery = $dbh->prepare($query) or die "Cannot prepare query \n";
	$rv = $sqlquery->execute or die "Cannot execute query \n";
	@row = $sqlquery->fetchrow_array();
	$junc_count = 0;
	@temp_array = ();

	# Load all junctions that start and end in the exon into arrays
	while (defined $row[0])
	{	
	 # check if junction overlaps with another junction
	$j = 0;
	$overlap = "F";
	$start_pos = $row[2] + $row[5];
	$end_pos = $row[3] - $row[6];
	while (($j < $junc_count) && ($overlap eq "F"))
	{
		$intron_start = $temp_array[$j][2] + $temp_array[$j][5];
		$intron_end = $temp_array[$j][3] - $temp_array[$j][6];
		if (($start_pos >= $intron_start) && ($start_pos <= $intron_end ))
			{ $overlap = "T"; }
		elsif (($end_pos >= $intron_start) && ($end_pos <= $intron_end ))
			{ $overlap = "T"; }
		elsif (($start_pos <= $intron_start) && ($end_pos >= $intron_end ))
			{ $overlap = "T"; }
		$j++;
	}


	if ($overlap eq "F")	# junction does not overlap, then add to temp_array
		{
		$temp_array[$junc_count] = [@row];
		$junc_count++;
		}
	@row = $sqlquery->fetchrow_array();

	}
	$sqlquery -> finish();
	$num_junc = $junc_count;
	$debug++;

	# Sort the array based on start position (column 2 )
	@junc_array = sort { $a->[2] <=> $b->[2] } @temp_array;


	if ($num_junc == 0)
		{ print OUTFILE "$_"; }
	else
		{
		$junc_count = 0;
		$exon_count = 0;
		$start_pos = $start_pos1;
		while ($junc_count < $num_junc)
			{ 
			$intron_start = $junc_array[$junc_count][2] + $junc_array[$junc_count][5];
			$intron_end = $junc_array[$junc_count][3] - $junc_array[$junc_count][6];
			if (defined $junc_array[$junc_count+1][0])
				{ $end_pos = $junc_array[$junc_count+1][2] + $junc_array[$junc_count+1][5]; }	# Set end_pos to start of next intron. Note: This assumes the next intron is valid
			else
				{ $end_pos = $end_pos1; }

			# have to subtract 25 so that the midpoint of the reads will fall within the range
			$start_pos25 = $start_pos - 25;
			$intron_start25 = $intron_start -25;
			$intron_end25 = $intron_end - 25;
			$end_pos25 = $end_pos - 25;

			# only look at reads that do not have junctions
			if ($samptype eq "All")
			{
			$query1 = "select count(*) from $short_read_table where (chromosome = \"$chrom1\") and (start_position >= $start_pos25) and (start_position <= $intron_start25) and (cigar = \"50M\")";
			$query2 = "select count(*) from $short_read_table where (chromosome = \"$chrom1\") and (start_position >= $intron_start25) and (start_position <= $intron_end25) and (cigar = \"50M\")";
			$query3 = "select count(*) from $short_read_table where (chromosome = \"$chrom1\") and (start_position >= $intron_end25) and (start_position <= $end_pos25) and (cigar = \"50M\")";
			}
			else
			{
			$query1 = "select count(*) from $short_read_table where (chromosome = \"$chrom1\") and (start_position >= $start_pos25) and (start_position <= $intron_start25) and (sample_type = \"$samptype\") and (cigar = \"50M\")";
			$query2 = "select count(*) from $short_read_table where (chromosome = \"$chrom1\") and (start_position >= $intron_start25) and (start_position <= $intron_end25) and (sample_type = \"$samptype\") and (cigar = \"50M\")";
			$query3 = "select count(*) from $short_read_table where (chromosome = \"$chrom1\") and (start_position >= $intron_end25) and (start_position <= $end_pos25) and (sample_type = \"$samptype\") and (cigar = \"50M\")";
			}

			$sqlquery1 = $dbh->prepare($query1) or die "Cannot prepare query \n";
			$sqlquery1->execute or die "Cannot execute query \n";
			@row1 = $sqlquery1->fetchrow_array();

			$sqlquery2 = $dbh->prepare($query2) or die "Cannot prepare query \n";
			$sqlquery2->execute or die "Cannot execute query \n";
			@row2 = $sqlquery2->fetchrow_array();

			$sqlquery3 = $dbh->prepare($query3) or die "Cannot prepare query \n";
			$sqlquery3->execute or die "Cannot execute query \n";
			@row3 = $sqlquery3->fetchrow_array();

			# calculate reads per kilobase of exon
			$exon_len1 = $intron_start - $start_pos;
			$intron_len = $intron_end - $intron_start;
			$exon_len2 = $end_pos - $intron_end;
			$rpke1 = $row1[0]/(($exon_len1)/1000);
			$rpke2 = $row2[0]/(($intron_len)/1000);
			$rpke3 = $row3[0]/(($exon_len2)/1000);

			$avg_rpke = ($rpke1 + $rpke3)/2;
			$adj_avg = $avg_rpke * ($intron_threshold / 100);
			if ($adj_avg >= $rpke2)		# If the rpke of the intron is less than the threshold, then save the exon into the arrays
			{
				# save exon data
				$exon_count++;
				$exon_start[$exon_count] = $start_pos;
				$exon_end[$exon_count] = $intron_start;
				$exon_reads[$exon_count] = $row1[0];
				$start_pos = $intron_end;
			}


			$junc_count++;
			} # while ($junc_count <= $num_junc)

		# save last exon	
		$exon_count++;
		$exon_start[$exon_count] = $start_pos;
		$exon_end[$exon_count] = $end_pos1;
		$exon_reads[$exon_count] = $row3[0];

		if ($exon_count <= 1)
			{ print OUTFILE "$_"; }
		else
			{ 
			$i = 1;
			while ($i <= $exon_count)
				{
				$exon_len = $exon_end[$i] - $exon_start[$i];
				print OUTFILE "$chrom1\t$exon_start[$i]\t$exon_end[$i]\t$exon_len\t$exon_reads[$i]\n";
				$i++;
				}

			} # if $exon_count <= 1

		} # if ($num_junc == 0 )

	} #if exon_len1 < $min_exon
}


close(INFILE);
close(OUTFILE);

print "Finished\n";
}
